@@ -29,7 +29,7 @@ import (
 const (
 	testversion = "v10.1.2-alpha.1.100+0123456789abcdef+SOMETHING"
 	expected    = "v10.1.2-alpha.1.100_0123456789abcdef_SOMETHING"
-	gcrPrefix   = "k8s.gcr.io"
+	gcrPrefix   = "registry.k8s.io"
 )
 
 func TestGetGenericImage(t *testing.T) {
