@@ -18,7 +18,7 @@ package kubeconfig
 
 import (
 	"fmt"
-	"io/ioutil"
+	"os"
 
 	"github.com/pkg/errors"
 
@@ -151,7 +151,7 @@ func EnsureAuthenticationInfoAreEmbedded(config *clientcmdapi.Config) error {
 	}
 
 	if len(authInfo.ClientCertificateData) == 0 && len(authInfo.ClientCertificate) != 0 {
-		clientCert, err := ioutil.ReadFile(authInfo.ClientCertificate)
+		clientCert, err := os.ReadFile(authInfo.ClientCertificate)
 		if err != nil {
 			return errors.Wrap(err, "error while reading client cert file defined in kubeconfig")
 		}
@@ -159,7 +159,7 @@ func EnsureAuthenticationInfoAreEmbedded(config *clientcmdapi.Config) error {
 		authInfo.ClientCertificate = ""
 	}
 	if len(authInfo.ClientKeyData) == 0 && len(authInfo.ClientKey) != 0 {
-		clientKey, err := ioutil.ReadFile(authInfo.ClientKey)
+		clientKey, err := os.ReadFile(authInfo.ClientKey)
 		if err != nil {
 			return errors.Wrap(err, "error while reading client key file defined in kubeconfig")
 		}
@@ -178,7 +178,7 @@ func EnsureCertificateAuthorityIsEmbedded(cluster *clientcmdapi.Cluster) error {
 	}
 
 	if len(cluster.CertificateAuthorityData) == 0 && len(cluster.CertificateAuthority) != 0 {
-		ca, err := ioutil.ReadFile(cluster.CertificateAuthority)
+		ca, err := os.ReadFile(cluster.CertificateAuthority)
 		if err != nil {
 			return errors.Wrap(err, "error while reading certificate authority file defined in kubeconfig")
 		}
