@@ -1,4 +1,4 @@
-#!/bin/bash
+#!/usr/bin/env bash
 
 # Copyright 2014 The Kubernetes Authors.
 #
@@ -14,38 +14,49 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-# GoFmt apparently is changing @ head...
+# This script checks whether the source code needs to be formatted or not by
+# `gofmt`. Run `hack/update-gofmt.sh` to actually format sources.
+#
+# Note: gofmt output can change between go versions.
+#
+# Usage: `hack/verify-gofmt.sh`.
 
 set -o errexit
 set -o nounset
 set -o pipefail
 
-KUBE_ROOT=$(dirname "${BASH_SOURCE}")/..
+KUBE_ROOT=$(dirname "${BASH_SOURCE[0]}")/..
 source "${KUBE_ROOT}/hack/lib/init.sh"
 
-kube::golang::verify_go_version
-
 cd "${KUBE_ROOT}"
 
+kube::golang::setup_env
+
 find_files() {
   find . -not \( \
       \( \
         -wholename './output' \
+        -o -wholename './.git' \
         -o -wholename './_output' \
         -o -wholename './_gopath' \
         -o -wholename './release' \
         -o -wholename './target' \
         -o -wholename '*/third_party/*' \
         -o -wholename '*/vendor/*' \
-        -o -wholename './staging' \
+        -o -wholename './staging/src/k8s.io/client-go/*vendor/*' \
+        -o -wholename '*/bindata.go' \
       \) -prune \
     \) -name '*.go'
 }
 
-GOFMT="gofmt -s -w"
-bad_files=$(find_files | xargs $GOFMT -l)
-if [[ -n "${bad_files}" ]]; then
-  echo "!!! '$GOFMT' needs to be run on the following files: "
-  echo "${bad_files}"
+# gofmt exits with non-zero exit code if it finds a problem unrelated to
+# formatting (e.g., a file does not parse correctly). Without "|| true" this
+# would have led to no useful error message from gofmt, because the script would
+# have failed before getting to the "echo" in the block below.
+diff=$(find_files | xargs gofmt -d -s 2>&1) || true
+if [[ -n "${diff}" ]]; then
+  echo "${diff}" >&2
+  echo >&2
+  echo "Run ./hack/update-gofmt.sh" >&2
   exit 1
 fi
