@@ -23,20 +23,16 @@ import (
 
 	v1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/util/runtime"
 	"k8s.io/apimachinery/pkg/util/wait"
 	"k8s.io/client-go/kubernetes/fake"
-	"k8s.io/client-go/tools/record"
 	kubepod "k8s.io/kubernetes/pkg/kubelet/pod"
 	"k8s.io/kubernetes/pkg/kubelet/prober/results"
 	"k8s.io/kubernetes/pkg/kubelet/status"
 	statustest "k8s.io/kubernetes/pkg/kubelet/status/testing"
 	"k8s.io/kubernetes/pkg/probe"
-	"k8s.io/utils/exec"
 )
 
 func init() {
-	runtime.ReallyCrash = true
 }
 
 func TestDoProbe(t *testing.T) {
@@ -296,27 +292,6 @@ func TestCleanUp(t *testing.T) {
 	}
 }
 
-func TestHandleCrash(t *testing.T) {
-	runtime.ReallyCrash = false // Test that we *don't* really crash.
-
-	m := newTestManager()
-	w := newTestWorker(m, readiness, v1.Probe{})
-	m.statusManager.SetPodStatus(w.pod, getTestRunningStatus())
-
-	expectContinue(t, w, w.doProbe(), "Initial successful probe.")
-	expectResult(t, w, results.Success, "Initial successful probe.")
-
-	// Prober starts crashing.
-	m.prober = &prober{
-		recorder: &record.FakeRecorder{},
-		exec:     crashingExecProber{},
-	}
-
-	// doProbe should recover from the crash, and keep going.
-	expectContinue(t, w, w.doProbe(), "Crashing probe.")
-	expectResult(t, w, results.Success, "Crashing probe unchanged.")
-}
-
 func expectResult(t *testing.T, w *worker, expectedResult results.Result, msg string) {
 	result, ok := resultsManager(w.probeManager, w.probeType).Get(w.containerID)
 	if !ok {
@@ -345,12 +320,6 @@ func resultsManager(m *manager, probeType probeType) results.Manager {
 	panic(fmt.Errorf("Unhandled case: %v", probeType))
 }
 
-type crashingExecProber struct{}
-
-func (p crashingExecProber) Probe(_ exec.Cmd) (probe.Result, string, error) {
-	panic("Intentional Probe crash.")
-}
-
 func TestOnHoldOnLivenessOrStartupCheckFailure(t *testing.T) {
 	m := newTestManager()
 
