@@ -92,10 +92,8 @@ func TestCSIDriverPrepareForCreate(t *testing.T) {
 	requiresRepublish := true
 
 	tests := []struct {
-		name                    string
-		withCapacity            bool
-		withInline              bool
-		withServiceAccountToken bool
+		name       string
+		withInline bool
 	}{
 		{
 			name:       "inline enabled",
@@ -105,29 +103,11 @@ func TestCSIDriverPrepareForCreate(t *testing.T) {
 			name:       "inline disabled",
 			withInline: false,
 		},
-		{
-			name:         "capacity enabled",
-			withCapacity: true,
-		},
-		{
-			name:         "capacity disabled",
-			withCapacity: false,
-		},
-		{
-			name:                    "serviceAccountToken enabled",
-			withServiceAccountToken: true,
-		},
-		{
-			name:                    "serviceAccountToken disabled",
-			withServiceAccountToken: false,
-		},
 	}
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
-			defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.CSIStorageCapacity, test.withCapacity)()
 			defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.CSIInlineVolume, test.withInline)()
-			defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.CSIServiceAccountToken, test.withServiceAccountToken)()
 
 			csiDriver := &storage.CSIDriver{
 				ObjectMeta: metav1.ObjectMeta{
@@ -149,14 +129,8 @@ func TestCSIDriverPrepareForCreate(t *testing.T) {
 			if len(errs) != 0 {
 				t.Errorf("unexpected validating errors: %v", errs)
 			}
-			if test.withCapacity {
-				if csiDriver.Spec.StorageCapacity == nil || *csiDriver.Spec.StorageCapacity != storageCapacity {
-					t.Errorf("StorageCapacity modified: %v", csiDriver.Spec.StorageCapacity)
-				}
-			} else {
-				if csiDriver.Spec.StorageCapacity != nil {
-					t.Errorf("StorageCapacity not stripped: %v", csiDriver.Spec.StorageCapacity)
-				}
+			if csiDriver.Spec.StorageCapacity == nil || *csiDriver.Spec.StorageCapacity != storageCapacity {
+				t.Errorf("StorageCapacity modified: %v", csiDriver.Spec.StorageCapacity)
 			}
 			if test.withInline {
 				if len(csiDriver.Spec.VolumeLifecycleModes) != 1 {
@@ -167,21 +141,6 @@ func TestCSIDriverPrepareForCreate(t *testing.T) {
 					t.Errorf("VolumeLifecycleModes not stripped: %v", csiDriver.Spec)
 				}
 			}
-			if test.withServiceAccountToken {
-				if csiDriver.Spec.TokenRequests == nil {
-					t.Errorf("TokenRequests modified: %v", csiDriver.Spec)
-				}
-				if csiDriver.Spec.RequiresRepublish == nil {
-					t.Errorf("RequiresRepublish modified: %v", csiDriver.Spec)
-				}
-			} else {
-				if csiDriver.Spec.TokenRequests != nil {
-					t.Errorf("TokenRequests stripped: %v", csiDriver.Spec)
-				}
-				if csiDriver.Spec.RequiresRepublish != nil {
-					t.Errorf("RequiresRepublish stripped: %v", csiDriver.Spec)
-				}
-			}
 		})
 	}
 }
@@ -227,7 +186,6 @@ func TestCSIDriverPrepareForUpdate(t *testing.T) {
 	enabled := true
 	disabled := false
 	gcp := "gcp"
-	vault := "vault"
 	driverWithCapacityEnabled := &storage.CSIDriver{
 		ObjectMeta: metav1.ObjectMeta{
 			Name: "foo",
@@ -253,45 +211,27 @@ func TestCSIDriverPrepareForUpdate(t *testing.T) {
 			RequiresRepublish: &enabled,
 		},
 	}
-	driverWithServiceAccountTokenVault := &storage.CSIDriver{
-		ObjectMeta: metav1.ObjectMeta{
-			Name: "foo",
-		},
-		Spec: storage.CSIDriverSpec{
-			TokenRequests:     []storage.TokenRequest{{Audience: vault}},
-			RequiresRepublish: &enabled,
-		},
-	}
 
 	resultPersistent := []storage.VolumeLifecycleMode{storage.VolumeLifecyclePersistent}
 
 	tests := []struct {
-		name                          string
-		old, update                   *storage.CSIDriver
-		csiStorageCapacityEnabled     bool
-		csiInlineVolumeEnabled        bool
-		csiServiceAccountTokenEnabled bool
-		wantCapacity                  *bool
-		wantModes                     []storage.VolumeLifecycleMode
-		wantTokenRequests             []storage.TokenRequest
-		wantRequiresRepublish         *bool
-		wantGeneration                int64
+		name                   string
+		old, update            *storage.CSIDriver
+		csiInlineVolumeEnabled bool
+		wantCapacity           *bool
+		wantModes              []storage.VolumeLifecycleMode
+		wantTokenRequests      []storage.TokenRequest
+		wantRequiresRepublish  *bool
+		wantGeneration         int64
 	}{
 		{
-			name:                      "capacity feature enabled, before: none, update: enabled",
-			csiStorageCapacityEnabled: true,
-			old:                       driverWithNothing,
-			update:                    driverWithCapacityEnabled,
-			wantCapacity:              &enabled,
-		},
-		{
-			name:         "capacity feature disabled, before: none, update: disabled",
+			name:         "capacity feature enabled, before: none, update: enabled",
 			old:          driverWithNothing,
-			update:       driverWithCapacityDisabled,
-			wantCapacity: nil,
+			update:       driverWithCapacityEnabled,
+			wantCapacity: &enabled,
 		},
 		{
-			name:         "capacity feature disabled, before: enabled, update: disabled",
+			name:         "capacity feature enabled, before: enabled, update: disabled",
 			old:          driverWithCapacityEnabled,
 			update:       driverWithCapacityDisabled,
 			wantCapacity: &disabled,
@@ -316,25 +256,9 @@ func TestCSIDriverPrepareForUpdate(t *testing.T) {
 			wantModes: resultPersistent,
 		},
 		{
-			name:                          "service account token feature enabled, before: none, update: audience=gcp",
-			csiServiceAccountTokenEnabled: true,
-			old:                           driverWithNothing,
-			update:                        driverWithServiceAccountTokenGCP,
-			wantTokenRequests:             []storage.TokenRequest{{Audience: gcp}},
-			wantRequiresRepublish:         &enabled,
-			wantGeneration:                1,
-		},
-		{
-			name:                  "service account token feature disabled, before: none, update: audience=gcp",
+			name:                  "service account token feature enabled, before: none, update: audience=gcp",
 			old:                   driverWithNothing,
 			update:                driverWithServiceAccountTokenGCP,
-			wantTokenRequests:     nil,
-			wantRequiresRepublish: nil,
-		},
-		{
-			name:                  "service account token feature disabled, before: audience=vault, update: audience=gcp",
-			old:                   driverWithServiceAccountTokenVault,
-			update:                driverWithServiceAccountTokenGCP,
 			wantTokenRequests:     []storage.TokenRequest{{Audience: gcp}},
 			wantRequiresRepublish: &enabled,
 			wantGeneration:        1,
@@ -343,9 +267,7 @@ func TestCSIDriverPrepareForUpdate(t *testing.T) {
 
 	for _, test := range tests {
 		t.Run(test.name, func(t *testing.T) {
-			defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.CSIStorageCapacity, test.csiStorageCapacityEnabled)()
 			defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.CSIInlineVolume, test.csiInlineVolumeEnabled)()
-			defer featuregatetesting.SetFeatureGateDuringTest(t, utilfeature.DefaultFeatureGate, features.CSIServiceAccountToken, test.csiServiceAccountTokenEnabled)()
 
 			csiDriver := test.update.DeepCopy()
 			Strategy.PrepareForUpdate(ctx, csiDriver, test.old)
