@@ -22,9 +22,10 @@ import (
 	"strings"
 	"testing"
 
+	"github.com/google/go-cmp/cmp"
 	"github.com/stretchr/testify/assert"
 	corev1 "k8s.io/api/core/v1"
-	"k8s.io/apimachinery/pkg/api/errors"
+	v1 "k8s.io/api/core/v1"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
 	"k8s.io/apimachinery/pkg/util/diff"
@@ -32,16 +33,11 @@ import (
 	admissiontesting "k8s.io/apiserver/pkg/admission/testing"
 	"k8s.io/client-go/informers"
 	"k8s.io/client-go/kubernetes/fake"
-	corev1listers "k8s.io/client-go/listers/core/v1"
-	"k8s.io/client-go/tools/cache"
 	api "k8s.io/kubernetes/pkg/apis/core"
+	v1defaults "k8s.io/kubernetes/pkg/apis/core/v1"
 	"k8s.io/kubernetes/pkg/controller"
 	kubelet "k8s.io/kubernetes/pkg/kubelet/types"
-)
-
-var (
-	deprecationDisabledBoundTokenVolume = false
-	deprecationEnabledBoundTokenVolume  = true
+	utilpointer "k8s.io/utils/pointer"
 )
 
 func TestIgnoresNonCreate(t *testing.T) {
@@ -165,59 +161,6 @@ func TestRejectsMirrorPodWithServiceAccountTokenVolumeProjections(t *testing.T)
 	}
 }
 
-func TestAssignsDefaultServiceAccountAndToleratesMissingAPIToken(t *testing.T) {
-	ns := "myns"
-
-	admit := NewServiceAccount()
-	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
-	admit.SetExternalKubeInformerFactory(informerFactory)
-	admit.MountServiceAccountToken = true
-	admit.RequireAPIToken = false
-
-	// Add the default service account for the ns into the cache
-	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      DefaultServiceAccountName,
-			Namespace: ns,
-		},
-	})
-
-	pod := &api.Pod{}
-	attrs := admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
-	err := admissiontesting.WithReinvocationTesting(t, admit).Admit(context.TODO(), attrs, nil)
-	if err != nil {
-		t.Errorf("Unexpected error: %v", err)
-	}
-	if pod.Spec.ServiceAccountName != DefaultServiceAccountName {
-		t.Errorf("Expected service account %s assigned, got %s", DefaultServiceAccountName, pod.Spec.ServiceAccountName)
-	}
-}
-
-func TestAssignsDefaultServiceAccountAndRejectsMissingAPIToken(t *testing.T) {
-	ns := "myns"
-
-	admit := NewServiceAccount()
-	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
-	admit.SetExternalKubeInformerFactory(informerFactory)
-	admit.MountServiceAccountToken = true
-	admit.RequireAPIToken = true
-
-	// Add the default service account for the ns into the cache
-	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      DefaultServiceAccountName,
-			Namespace: ns,
-		},
-	})
-
-	pod := &api.Pod{}
-	attrs := admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
-	err := admissiontesting.WithReinvocationTesting(t, admit).Admit(context.TODO(), attrs, nil)
-	if err == nil || !errors.IsServerTimeout(err) {
-		t.Errorf("Expected server timeout error for missing API token: %v", err)
-	}
-}
-
 func TestAssignsDefaultServiceAccountAndBoundTokenWithNoSecretTokens(t *testing.T) {
 	ns := "myns"
 
@@ -225,8 +168,6 @@ func TestAssignsDefaultServiceAccountAndBoundTokenWithNoSecretTokens(t *testing.
 	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
 	admit.SetExternalKubeInformerFactory(informerFactory)
 	admit.MountServiceAccountToken = true
-	admit.RequireAPIToken = true
-	admit.boundServiceAccountTokenVolume = true
 
 	// Add the default service account for the ns into the cache
 	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
@@ -236,11 +177,16 @@ func TestAssignsDefaultServiceAccountAndBoundTokenWithNoSecretTokens(t *testing.
 		},
 	})
 
-	pod := &api.Pod{
-		Spec: api.PodSpec{
-			Containers: []api.Container{{}},
+	v1PodIn := &v1.Pod{
+		Spec: v1.PodSpec{
+			Containers: []v1.Container{{}},
 		},
 	}
+	v1defaults.SetObjectDefaults_Pod(v1PodIn)
+	pod := &api.Pod{}
+	if err := v1defaults.Convert_v1_Pod_To_core_Pod(v1PodIn, pod, nil); err != nil {
+		t.Fatal(err)
+	}
 	attrs := admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
 	err := admissiontesting.WithReinvocationTesting(t, admit).Admit(context.TODO(), attrs, nil)
 	if err != nil {
@@ -256,6 +202,7 @@ func TestAssignsDefaultServiceAccountAndBoundTokenWithNoSecretTokens(t *testing.
 					{ConfigMap: &api.ConfigMapProjection{LocalObjectReference: api.LocalObjectReference{Name: "kube-root-ca.crt"}, Items: []api.KeyToPath{{Key: "ca.crt", Path: "ca.crt"}}}},
 					{DownwardAPI: &api.DownwardAPIProjection{Items: []api.DownwardAPIVolumeFile{{Path: "namespace", FieldRef: &api.ObjectFieldSelector{APIVersion: "v1", FieldPath: "metadata.namespace"}}}}},
 				},
+				DefaultMode: utilpointer.Int32(0644),
 			},
 		},
 	}}
@@ -283,6 +230,17 @@ func TestAssignsDefaultServiceAccountAndBoundTokenWithNoSecretTokens(t *testing.
 	if !reflect.DeepEqual(expectedVolumeMounts, pod.Spec.Containers[0].VolumeMounts) {
 		t.Errorf("unexpected volumes: %s", diff.ObjectReflectDiff(expectedVolumeMounts, pod.Spec.Containers[0].VolumeMounts))
 	}
+
+	// ensure result converted to v1 matches defaulted object
+	v1PodOut := &v1.Pod{}
+	if err := v1defaults.Convert_core_Pod_To_v1_Pod(pod, v1PodOut, nil); err != nil {
+		t.Fatal(err)
+	}
+	v1PodOutDefaulted := v1PodOut.DeepCopy()
+	v1defaults.SetObjectDefaults_Pod(v1PodOutDefaulted)
+	if !reflect.DeepEqual(v1PodOut, v1PodOutDefaulted) {
+		t.Error(cmp.Diff(v1PodOut, v1PodOutDefaulted))
+	}
 }
 
 func TestFetchesUncachedServiceAccount(t *testing.T) {
@@ -300,7 +258,6 @@ func TestFetchesUncachedServiceAccount(t *testing.T) {
 	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
 	admit.SetExternalKubeInformerFactory(informerFactory)
 	admit.client = client
-	admit.RequireAPIToken = false
 
 	pod := &api.Pod{}
 	attrs := admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
@@ -333,222 +290,181 @@ func TestDeniesInvalidServiceAccount(t *testing.T) {
 }
 
 func TestAutomountsAPIToken(t *testing.T) {
-	testBoundServiceAccountTokenVolumePhases(t, func(t *testing.T, applyFeatures func(*Plugin) *Plugin) {
 
-		admit := applyFeatures(NewServiceAccount())
-		informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
-		admit.SetExternalKubeInformerFactory(informerFactory)
-		admit.generateName = testGenerateName
-		admit.MountServiceAccountToken = true
-		admit.RequireAPIToken = true
+	admit := NewServiceAccount()
+	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
+	admit.SetExternalKubeInformerFactory(informerFactory)
+	admit.generateName = testGenerateName
+	admit.MountServiceAccountToken = true
 
-		ns := "myns"
-		serviceAccountName := DefaultServiceAccountName
-		serviceAccountUID := "12345"
+	ns := "myns"
+	serviceAccountName := DefaultServiceAccountName
+	serviceAccountUID := "12345"
 
-		tokenName := "token-name"
-		if admit.boundServiceAccountTokenVolume {
-			tokenName = generatedVolumeName
-		}
+	tokenName := generatedVolumeName
 
-		expectedVolume := admit.createVolume(tokenName, tokenName)
-		expectedVolumeMount := api.VolumeMount{
-			Name:      tokenName,
-			ReadOnly:  true,
-			MountPath: DefaultAPITokenMountPath,
-		}
-		// Add the default service account for the ns with a token into the cache
-		informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
-			ObjectMeta: metav1.ObjectMeta{
-				Name:      serviceAccountName,
-				Namespace: ns,
-				UID:       types.UID(serviceAccountUID),
-			},
-			Secrets: []corev1.ObjectReference{
-				{Name: tokenName},
-			},
-		})
-		// Add a token for the service account into the cache
-		informerFactory.Core().V1().Secrets().Informer().GetStore().Add(&corev1.Secret{
-			ObjectMeta: metav1.ObjectMeta{
-				Name:      tokenName,
-				Namespace: ns,
-				Annotations: map[string]string{
-					corev1.ServiceAccountNameKey: serviceAccountName,
-					corev1.ServiceAccountUIDKey:  serviceAccountUID,
-				},
-			},
-			Type: corev1.SecretTypeServiceAccountToken,
-			Data: map[string][]byte{
-				api.ServiceAccountTokenKey: []byte("token-data"),
-			},
-		})
-
-		pod := &api.Pod{
-			Spec: api.PodSpec{
-				Containers: []api.Container{
-					{},
-				},
-			},
-		}
-		attrs := admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
-		err := admissiontesting.WithReinvocationTesting(t, admit).Admit(context.TODO(), attrs, nil)
-		if err != nil {
-			t.Errorf("Unexpected error: %v", err)
-		}
-		if pod.Spec.ServiceAccountName != DefaultServiceAccountName {
-			t.Errorf("Expected service account %s assigned, got %s", DefaultServiceAccountName, pod.Spec.ServiceAccountName)
-		}
-		if len(pod.Spec.Volumes) != 1 {
-			t.Fatalf("Expected 1 volume, got %d", len(pod.Spec.Volumes))
-		}
-		if !reflect.DeepEqual(expectedVolume, pod.Spec.Volumes[0]) {
-			t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolume, pod.Spec.Volumes[0])
-		}
-		if len(pod.Spec.Containers[0].VolumeMounts) != 1 {
-			t.Fatalf("Expected 1 volume mount, got %d", len(pod.Spec.Containers[0].VolumeMounts))
-		}
-		if !reflect.DeepEqual(expectedVolumeMount, pod.Spec.Containers[0].VolumeMounts[0]) {
-			t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolumeMount, pod.Spec.Containers[0].VolumeMounts[0])
-		}
-
-		// testing InitContainers
-		pod = &api.Pod{
-			Spec: api.PodSpec{
-				InitContainers: []api.Container{
-					{},
-				},
-			},
-		}
-		attrs = admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
-		if err := admissiontesting.WithReinvocationTesting(t, admit).Admit(context.TODO(), attrs, nil); err != nil {
-			t.Errorf("Unexpected error: %v", err)
-		}
-		if pod.Spec.ServiceAccountName != DefaultServiceAccountName {
-			t.Errorf("Expected service account %s assigned, got %s", DefaultServiceAccountName, pod.Spec.ServiceAccountName)
-		}
-		if len(pod.Spec.Volumes) != 1 {
-			t.Fatalf("Expected 1 volume, got %d", len(pod.Spec.Volumes))
-		}
-		if !reflect.DeepEqual(expectedVolume, pod.Spec.Volumes[0]) {
-			t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolume, pod.Spec.Volumes[0])
-		}
-		if len(pod.Spec.InitContainers[0].VolumeMounts) != 1 {
-			t.Fatalf("Expected 1 volume mount, got %d", len(pod.Spec.InitContainers[0].VolumeMounts))
-		}
-		if !reflect.DeepEqual(expectedVolumeMount, pod.Spec.InitContainers[0].VolumeMounts[0]) {
-			t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolumeMount, pod.Spec.InitContainers[0].VolumeMounts[0])
-		}
+	expectedVolume := api.Volume{
+		Name: tokenName,
+		VolumeSource: api.VolumeSource{
+			Projected: TokenVolumeSource(),
+		},
+	}
+	expectedVolumeMount := api.VolumeMount{
+		Name:      tokenName,
+		ReadOnly:  true,
+		MountPath: DefaultAPITokenMountPath,
+	}
+	// Add the default service account for the ns with a token into the cache
+	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      serviceAccountName,
+			Namespace: ns,
+			UID:       types.UID(serviceAccountUID),
+		},
 	})
+
+	pod := &api.Pod{
+		Spec: api.PodSpec{
+			Containers: []api.Container{
+				{},
+			},
+		},
+	}
+	attrs := admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
+	err := admissiontesting.WithReinvocationTesting(t, admit).Admit(context.TODO(), attrs, nil)
+	if err != nil {
+		t.Errorf("Unexpected error: %v", err)
+	}
+	if pod.Spec.ServiceAccountName != DefaultServiceAccountName {
+		t.Errorf("Expected service account %s assigned, got %s", DefaultServiceAccountName, pod.Spec.ServiceAccountName)
+	}
+	if len(pod.Spec.Volumes) != 1 {
+		t.Fatalf("Expected 1 volume, got %d", len(pod.Spec.Volumes))
+	}
+	if !reflect.DeepEqual(expectedVolume, pod.Spec.Volumes[0]) {
+		t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolume, pod.Spec.Volumes[0])
+	}
+	if len(pod.Spec.Containers[0].VolumeMounts) != 1 {
+		t.Fatalf("Expected 1 volume mount, got %d", len(pod.Spec.Containers[0].VolumeMounts))
+	}
+	if !reflect.DeepEqual(expectedVolumeMount, pod.Spec.Containers[0].VolumeMounts[0]) {
+		t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolumeMount, pod.Spec.Containers[0].VolumeMounts[0])
+	}
+
+	// testing InitContainers
+	pod = &api.Pod{
+		Spec: api.PodSpec{
+			InitContainers: []api.Container{
+				{},
+			},
+		},
+	}
+	attrs = admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
+	if err := admissiontesting.WithReinvocationTesting(t, admit).Admit(context.TODO(), attrs, nil); err != nil {
+		t.Errorf("Unexpected error: %v", err)
+	}
+	if pod.Spec.ServiceAccountName != DefaultServiceAccountName {
+		t.Errorf("Expected service account %s assigned, got %s", DefaultServiceAccountName, pod.Spec.ServiceAccountName)
+	}
+	if len(pod.Spec.Volumes) != 1 {
+		t.Fatalf("Expected 1 volume, got %d", len(pod.Spec.Volumes))
+	}
+	if !reflect.DeepEqual(expectedVolume, pod.Spec.Volumes[0]) {
+		t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolume, pod.Spec.Volumes[0])
+	}
+	if len(pod.Spec.InitContainers[0].VolumeMounts) != 1 {
+		t.Fatalf("Expected 1 volume mount, got %d", len(pod.Spec.InitContainers[0].VolumeMounts))
+	}
+	if !reflect.DeepEqual(expectedVolumeMount, pod.Spec.InitContainers[0].VolumeMounts[0]) {
+		t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolumeMount, pod.Spec.InitContainers[0].VolumeMounts[0])
+	}
 }
 
 func TestRespectsExistingMount(t *testing.T) {
-	testBoundServiceAccountTokenVolumePhases(t, func(t *testing.T, applyFeatures func(*Plugin) *Plugin) {
-		ns := "myns"
-		tokenName := "token-name"
-		serviceAccountName := DefaultServiceAccountName
-		serviceAccountUID := "12345"
+	ns := "myns"
+	serviceAccountName := DefaultServiceAccountName
+	serviceAccountUID := "12345"
 
-		expectedVolumeMount := api.VolumeMount{
-			Name:      "my-custom-mount",
-			ReadOnly:  false,
-			MountPath: DefaultAPITokenMountPath,
-		}
+	expectedVolumeMount := api.VolumeMount{
+		Name:      "my-custom-mount",
+		ReadOnly:  false,
+		MountPath: DefaultAPITokenMountPath,
+	}
 
-		admit := applyFeatures(NewServiceAccount())
-		informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
-		admit.SetExternalKubeInformerFactory(informerFactory)
-		admit.MountServiceAccountToken = true
-		admit.RequireAPIToken = true
+	admit := NewServiceAccount()
+	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
+	admit.SetExternalKubeInformerFactory(informerFactory)
+	admit.MountServiceAccountToken = true
 
-		// Add the default service account for the ns with a token into the cache
-		informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
-			ObjectMeta: metav1.ObjectMeta{
-				Name:      serviceAccountName,
-				Namespace: ns,
-				UID:       types.UID(serviceAccountUID),
-			},
-			Secrets: []corev1.ObjectReference{
-				{Name: tokenName},
-			},
-		})
-		// Add a token for the service account into the cache
-		informerFactory.Core().V1().Secrets().Informer().GetStore().Add(&corev1.Secret{
-			ObjectMeta: metav1.ObjectMeta{
-				Name:      tokenName,
-				Namespace: ns,
-				Annotations: map[string]string{
-					corev1.ServiceAccountNameKey: serviceAccountName,
-					corev1.ServiceAccountUIDKey:  serviceAccountUID,
-				},
-			},
-			Type: corev1.SecretTypeServiceAccountToken,
-			Data: map[string][]byte{
-				corev1.ServiceAccountTokenKey: []byte("token-data"),
-			},
-		})
-
-		// Define a pod with a container that already mounts a volume at the API token path
-		// Admission should respect that
-		// Additionally, no volume should be created if no container is going to use it
-		pod := &api.Pod{
-			Spec: api.PodSpec{
-				Containers: []api.Container{
-					{
-						VolumeMounts: []api.VolumeMount{
-							expectedVolumeMount,
-						},
-					},
-				},
-			},
-		}
-		attrs := admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
-		err := admissiontesting.WithReinvocationTesting(t, admit).Admit(context.TODO(), attrs, nil)
-		if err != nil {
-			t.Errorf("Unexpected error: %v", err)
-		}
-		if pod.Spec.ServiceAccountName != DefaultServiceAccountName {
-			t.Errorf("Expected service account %s assigned, got %s", DefaultServiceAccountName, pod.Spec.ServiceAccountName)
-		}
-		if len(pod.Spec.Volumes) != 0 {
-			t.Fatalf("Expected 0 volumes (shouldn't create a volume for a secret we don't need), got %d", len(pod.Spec.Volumes))
-		}
-		if len(pod.Spec.Containers[0].VolumeMounts) != 1 {
-			t.Fatalf("Expected 1 volume mount, got %d", len(pod.Spec.Containers[0].VolumeMounts))
-		}
-		if !reflect.DeepEqual(expectedVolumeMount, pod.Spec.Containers[0].VolumeMounts[0]) {
-			t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolumeMount, pod.Spec.Containers[0].VolumeMounts[0])
-		}
-
-		// check init containers
-		pod = &api.Pod{
-			Spec: api.PodSpec{
-				InitContainers: []api.Container{
-					{
-						VolumeMounts: []api.VolumeMount{
-							expectedVolumeMount,
-						},
-					},
-				},
-			},
-		}
-		attrs = admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
-		if err := admissiontesting.WithReinvocationTesting(t, admit).Admit(context.TODO(), attrs, nil); err != nil {
-			t.Errorf("Unexpected error: %v", err)
-		}
-		if pod.Spec.ServiceAccountName != DefaultServiceAccountName {
-			t.Errorf("Expected service account %s assigned, got %s", DefaultServiceAccountName, pod.Spec.ServiceAccountName)
-		}
-		if len(pod.Spec.Volumes) != 0 {
-			t.Fatalf("Expected 0 volumes (shouldn't create a volume for a secret we don't need), got %d", len(pod.Spec.Volumes))
-		}
-		if len(pod.Spec.InitContainers[0].VolumeMounts) != 1 {
-			t.Fatalf("Expected 1 volume mount, got %d", len(pod.Spec.InitContainers[0].VolumeMounts))
-		}
-		if !reflect.DeepEqual(expectedVolumeMount, pod.Spec.InitContainers[0].VolumeMounts[0]) {
-			t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolumeMount, pod.Spec.InitContainers[0].VolumeMounts[0])
-		}
+	// Add the default service account for the ns with a token into the cache
+	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
+		ObjectMeta: metav1.ObjectMeta{
+			Name:      serviceAccountName,
+			Namespace: ns,
+			UID:       types.UID(serviceAccountUID),
+		},
 	})
+
+	// Define a pod with a container that already mounts a volume at the API token path
+	// Admission should respect that
+	// Additionally, no volume should be created if no container is going to use it
+	pod := &api.Pod{
+		Spec: api.PodSpec{
+			Containers: []api.Container{
+				{
+					VolumeMounts: []api.VolumeMount{
+						expectedVolumeMount,
+					},
+				},
+			},
+		},
+	}
+	attrs := admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
+	err := admissiontesting.WithReinvocationTesting(t, admit).Admit(context.TODO(), attrs, nil)
+	if err != nil {
+		t.Errorf("Unexpected error: %v", err)
+	}
+	if pod.Spec.ServiceAccountName != DefaultServiceAccountName {
+		t.Errorf("Expected service account %s assigned, got %s", DefaultServiceAccountName, pod.Spec.ServiceAccountName)
+	}
+	if len(pod.Spec.Volumes) != 0 {
+		t.Fatalf("Expected 0 volumes (shouldn't create a volume for a secret we don't need), got %d", len(pod.Spec.Volumes))
+	}
+	if len(pod.Spec.Containers[0].VolumeMounts) != 1 {
+		t.Fatalf("Expected 1 volume mount, got %d", len(pod.Spec.Containers[0].VolumeMounts))
+	}
+	if !reflect.DeepEqual(expectedVolumeMount, pod.Spec.Containers[0].VolumeMounts[0]) {
+		t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolumeMount, pod.Spec.Containers[0].VolumeMounts[0])
+	}
+
+	// check init containers
+	pod = &api.Pod{
+		Spec: api.PodSpec{
+			InitContainers: []api.Container{
+				{
+					VolumeMounts: []api.VolumeMount{
+						expectedVolumeMount,
+					},
+				},
+			},
+		},
+	}
+	attrs = admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
+	if err := admissiontesting.WithReinvocationTesting(t, admit).Admit(context.TODO(), attrs, nil); err != nil {
+		t.Errorf("Unexpected error: %v", err)
+	}
+	if pod.Spec.ServiceAccountName != DefaultServiceAccountName {
+		t.Errorf("Expected service account %s assigned, got %s", DefaultServiceAccountName, pod.Spec.ServiceAccountName)
+	}
+	if len(pod.Spec.Volumes) != 0 {
+		t.Fatalf("Expected 0 volumes (shouldn't create a volume for a secret we don't need), got %d", len(pod.Spec.Volumes))
+	}
+	if len(pod.Spec.InitContainers[0].VolumeMounts) != 1 {
+		t.Fatalf("Expected 1 volume mount, got %d", len(pod.Spec.InitContainers[0].VolumeMounts))
+	}
+	if !reflect.DeepEqual(expectedVolumeMount, pod.Spec.InitContainers[0].VolumeMounts[0]) {
+		t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolumeMount, pod.Spec.InitContainers[0].VolumeMounts[0])
+	}
 }
 
 func TestAllowsReferencedSecret(t *testing.T) {
@@ -558,7 +474,6 @@ func TestAllowsReferencedSecret(t *testing.T) {
 	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
 	admit.SetExternalKubeInformerFactory(informerFactory)
 	admit.LimitSecretReferences = true
-	admit.RequireAPIToken = false
 
 	// Add the default service account for the ns with a secret reference into the cache
 	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
@@ -639,7 +554,6 @@ func TestRejectsUnreferencedSecretVolumes(t *testing.T) {
 	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
 	admit.SetExternalKubeInformerFactory(informerFactory)
 	admit.LimitSecretReferences = true
-	admit.RequireAPIToken = false
 
 	// Add the default service account for the ns into the cache
 	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
@@ -717,7 +631,6 @@ func TestAllowUnreferencedSecretVolumesForPermissiveSAs(t *testing.T) {
 	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
 	admit.SetExternalKubeInformerFactory(informerFactory)
 	admit.LimitSecretReferences = false
-	admit.RequireAPIToken = false
 
 	// Add the default service account for the ns into the cache
 	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
@@ -749,7 +662,6 @@ func TestAllowsReferencedImagePullSecrets(t *testing.T) {
 	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
 	admit.SetExternalKubeInformerFactory(informerFactory)
 	admit.LimitSecretReferences = true
-	admit.RequireAPIToken = false
 
 	// Add the default service account for the ns with a secret reference into the cache
 	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
@@ -781,7 +693,6 @@ func TestRejectsUnreferencedImagePullSecrets(t *testing.T) {
 	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
 	admit.SetExternalKubeInformerFactory(informerFactory)
 	admit.LimitSecretReferences = true
-	admit.RequireAPIToken = false
 
 	// Add the default service account for the ns into the cache
 	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
@@ -810,7 +721,6 @@ func TestDoNotAddImagePullSecrets(t *testing.T) {
 	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
 	admit.SetExternalKubeInformerFactory(informerFactory)
 	admit.LimitSecretReferences = true
-	admit.RequireAPIToken = false
 
 	// Add the default service account for the ns with a secret reference into the cache
 	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
@@ -847,7 +757,6 @@ func TestAddImagePullSecrets(t *testing.T) {
 	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
 	admit.SetExternalKubeInformerFactory(informerFactory)
 	admit.LimitSecretReferences = true
-	admit.RequireAPIToken = false
 
 	sa := &corev1.ServiceAccount{
 		ObjectMeta: metav1.ObjectMeta{
@@ -859,6 +768,11 @@ func TestAddImagePullSecrets(t *testing.T) {
 			{Name: "bar"},
 		},
 	}
+	originalSA := sa.DeepCopy()
+	expected := []api.LocalObjectReference{
+		{Name: "foo"},
+		{Name: "bar"},
+	}
 	// Add the default service account for the ns with a secret reference into the cache
 	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(sa)
 
@@ -869,356 +783,16 @@ func TestAddImagePullSecrets(t *testing.T) {
 		t.Errorf("Unexpected error: %v", err)
 	}
 
-	assert.EqualValues(t, sa.ImagePullSecrets, pod.Spec.ImagePullSecrets, "expected %v, got %v", sa.ImagePullSecrets, pod.Spec.ImagePullSecrets)
+	assert.EqualValues(t, expected, pod.Spec.ImagePullSecrets, "expected %v, got %v", expected, pod.Spec.ImagePullSecrets)
 
 	pod.Spec.ImagePullSecrets[1] = api.LocalObjectReference{Name: "baz"}
-	if reflect.DeepEqual(sa.ImagePullSecrets, pod.Spec.ImagePullSecrets) {
+	if !reflect.DeepEqual(originalSA, sa) {
 		t.Errorf("accidentally mutated the ServiceAccount.ImagePullSecrets: %v", sa.ImagePullSecrets)
 	}
 }
 
-func TestMultipleReferencedSecrets(t *testing.T) {
-	var (
-		ns                 = "myns"
-		serviceAccountName = "mysa"
-		serviceAccountUID  = "mysauid"
-		token1             = "token1"
-		token2             = "token2"
-	)
-
-	admit := NewServiceAccount()
-	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
-	admit.SetExternalKubeInformerFactory(informerFactory)
-	admit.MountServiceAccountToken = true
-	admit.RequireAPIToken = true
-
-	sa := &corev1.ServiceAccount{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      serviceAccountName,
-			UID:       types.UID(serviceAccountUID),
-			Namespace: ns,
-		},
-		Secrets: []corev1.ObjectReference{
-			{Name: token1},
-			{Name: token2},
-		},
-	}
-	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(sa)
-
-	// Add two tokens for the service account into the cache.
-	informerFactory.Core().V1().Secrets().Informer().GetStore().Add(&corev1.Secret{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      token2,
-			Namespace: ns,
-			Annotations: map[string]string{
-				api.ServiceAccountNameKey: serviceAccountName,
-				api.ServiceAccountUIDKey:  serviceAccountUID,
-			},
-		},
-		Type: corev1.SecretTypeServiceAccountToken,
-		Data: map[string][]byte{
-			api.ServiceAccountTokenKey: []byte("token-data"),
-		},
-	})
-	informerFactory.Core().V1().Secrets().Informer().GetStore().Add(&corev1.Secret{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      token1,
-			Namespace: ns,
-			Annotations: map[string]string{
-				api.ServiceAccountNameKey: serviceAccountName,
-				api.ServiceAccountUIDKey:  serviceAccountUID,
-			},
-		},
-		Type: corev1.SecretTypeServiceAccountToken,
-		Data: map[string][]byte{
-			api.ServiceAccountTokenKey: []byte("token-data"),
-		},
-	})
-
-	pod := &api.Pod{
-		Spec: api.PodSpec{
-			ServiceAccountName: serviceAccountName,
-			Containers: []api.Container{
-				{Name: "container-1"},
-			},
-		},
-	}
-
-	attrs := admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
-	if err := admissiontesting.WithReinvocationTesting(t, admit).Admit(context.TODO(), attrs, nil); err != nil {
-		t.Fatal(err)
-	}
-
-	if n := len(pod.Spec.Volumes); n != 1 {
-		t.Fatalf("expected 1 volume mount, got %d", n)
-	}
-	if name := pod.Spec.Volumes[0].Name; name != token1 {
-		t.Errorf("expected first referenced secret to be mounted, got %q", name)
-	}
-}
-
-func newSecret(secretType corev1.SecretType, namespace, name, serviceAccountName, serviceAccountUID string) *corev1.Secret {
-	return &corev1.Secret{
-		ObjectMeta: metav1.ObjectMeta{
-			Namespace: namespace,
-			Name:      name,
-			Annotations: map[string]string{
-				corev1.ServiceAccountNameKey: serviceAccountName,
-				corev1.ServiceAccountUIDKey:  serviceAccountUID,
-			},
-		},
-		Type: secretType,
-	}
-}
-
-func TestGetServiceAccountTokens(t *testing.T) {
-	testBoundServiceAccountTokenVolumePhases(t, func(t *testing.T, applyFeatures func(*Plugin) *Plugin) {
-		admit := applyFeatures(NewServiceAccount())
-		indexer := cache.NewIndexer(cache.MetaNamespaceKeyFunc, cache.Indexers{})
-		admit.secretLister = corev1listers.NewSecretLister(indexer)
-
-		ns := "namespace"
-		serviceAccountUID := "12345"
-
-		sa := &corev1.ServiceAccount{
-			ObjectMeta: metav1.ObjectMeta{
-				Name:      DefaultServiceAccountName,
-				Namespace: ns,
-				UID:       types.UID(serviceAccountUID),
-			},
-		}
-
-		nonSATokenSecret := newSecret(corev1.SecretTypeDockercfg, ns, "nonSATokenSecret", DefaultServiceAccountName, serviceAccountUID)
-		indexer.Add(nonSATokenSecret)
-
-		differentSAToken := newSecret(corev1.SecretTypeServiceAccountToken, ns, "differentSAToken", "someOtherSA", "someOtherUID")
-		indexer.Add(differentSAToken)
-
-		matchingSAToken := newSecret(corev1.SecretTypeServiceAccountToken, ns, "matchingSAToken", DefaultServiceAccountName, serviceAccountUID)
-		indexer.Add(matchingSAToken)
-
-		tokens, err := admit.getServiceAccountTokens(sa)
-		if err != nil {
-			t.Fatalf("unexpected error: %v", err)
-		}
-
-		if len(tokens) != 1 {
-			names := make([]string, 0, len(tokens))
-			for _, token := range tokens {
-				names = append(names, token.Name)
-			}
-			t.Fatalf("expected only 1 token, got %v", names)
-		}
-		if e, a := matchingSAToken.Name, tokens[0].Name; e != a {
-			t.Errorf("expected token %s, got %s", e, a)
-		}
-	})
-}
-
-func TestAutomountIsBackwardsCompatible(t *testing.T) {
-	ns := "myns"
-	tokenName := "token-name"
-	serviceAccountName := DefaultServiceAccountName
-	serviceAccountUID := "12345"
-	defaultTokenName := "default-token-abc123"
-
-	expectedVolume := api.Volume{
-		Name: defaultTokenName,
-		VolumeSource: api.VolumeSource{
-			Secret: &api.SecretVolumeSource{
-				SecretName: defaultTokenName,
-			},
-		},
-	}
-	expectedVolumeMount := api.VolumeMount{
-		Name:      defaultTokenName,
-		ReadOnly:  true,
-		MountPath: DefaultAPITokenMountPath,
-	}
-
-	admit := NewServiceAccount()
-	admit.generateName = testGenerateName
-	admit.boundServiceAccountTokenVolume = deprecationEnabledBoundTokenVolume
-
-	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
-	admit.SetExternalKubeInformerFactory(informerFactory)
-	admit.MountServiceAccountToken = true
-	admit.RequireAPIToken = true
-
-	// Add the default service account for the ns with a token into the cache
-	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      serviceAccountName,
-			Namespace: ns,
-			UID:       types.UID(serviceAccountUID),
-		},
-		Secrets: []corev1.ObjectReference{
-			{Name: tokenName},
-		},
-	})
-	// Add a token for the service account into the cache
-	informerFactory.Core().V1().Secrets().Informer().GetStore().Add(&corev1.Secret{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      tokenName,
-			Namespace: ns,
-			Annotations: map[string]string{
-				corev1.ServiceAccountNameKey: serviceAccountName,
-				corev1.ServiceAccountUIDKey:  serviceAccountUID,
-			},
-		},
-		Type: corev1.SecretTypeServiceAccountToken,
-		Data: map[string][]byte{
-			api.ServiceAccountTokenKey: []byte("token-data"),
-		},
-	})
-
-	pod := &api.Pod{
-		Spec: api.PodSpec{
-			Containers: []api.Container{
-				{
-					Name: "c-1",
-					VolumeMounts: []api.VolumeMount{
-						{
-							Name:      defaultTokenName,
-							MountPath: DefaultAPITokenMountPath,
-							ReadOnly:  true,
-						},
-					},
-				},
-			},
-			Volumes: []api.Volume{
-				{
-					Name: defaultTokenName,
-					VolumeSource: api.VolumeSource{
-						Secret: &api.SecretVolumeSource{
-							SecretName: defaultTokenName,
-						},
-					},
-				},
-			},
-		},
-	}
-	attrs := admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
-	err := admissiontesting.WithReinvocationTesting(t, admit).Admit(context.TODO(), attrs, nil)
-	if err != nil {
-		t.Errorf("Unexpected error: %v", err)
-	}
-	if pod.Spec.ServiceAccountName != DefaultServiceAccountName {
-		t.Errorf("Expected service account %s assigned, got %s", DefaultServiceAccountName, pod.Spec.ServiceAccountName)
-	}
-	_ = expectedVolume
-	_ = expectedVolumeMount
-	if len(pod.Spec.Volumes) != 1 {
-		t.Fatalf("Expected 1 volume, got %d", len(pod.Spec.Volumes))
-	}
-	if !reflect.DeepEqual(expectedVolume, pod.Spec.Volumes[0]) {
-		t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolume, pod.Spec.Volumes[0])
-	}
-	if len(pod.Spec.Containers[0].VolumeMounts) != 1 {
-		t.Fatalf("Expected 1 volume mount, got %d", len(pod.Spec.Containers[0].VolumeMounts))
-	}
-	if !reflect.DeepEqual(expectedVolumeMount, pod.Spec.Containers[0].VolumeMounts[0]) {
-		t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolumeMount, pod.Spec.Containers[0].VolumeMounts[0])
-	}
-}
-func TestServiceAccountNameWithDotMount(t *testing.T) {
-	ns := "myns"
-	tokenName := "token.name-123"
-	serviceAccountName := "token.name"
-	serviceAccountUID := "12345"
-
-	expectedVolume := api.Volume{
-		Name: "token-name-123",
-		VolumeSource: api.VolumeSource{
-			Secret: &api.SecretVolumeSource{
-				SecretName: "token.name-123",
-			},
-		},
-	}
-	expectedVolumeMount := api.VolumeMount{
-		Name:      "token-name-123",
-		ReadOnly:  true,
-		MountPath: DefaultAPITokenMountPath,
-	}
-
-	admit := NewServiceAccount()
-	informerFactory := informers.NewSharedInformerFactory(nil, controller.NoResyncPeriodFunc())
-	admit.SetExternalKubeInformerFactory(informerFactory)
-	admit.MountServiceAccountToken = true
-	admit.RequireAPIToken = true
-
-	informerFactory.Core().V1().ServiceAccounts().Informer().GetStore().Add(&corev1.ServiceAccount{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      serviceAccountName,
-			Namespace: ns,
-			UID:       types.UID(serviceAccountUID),
-		},
-		Secrets: []corev1.ObjectReference{
-			{Name: tokenName},
-		},
-	})
-
-	informerFactory.Core().V1().Secrets().Informer().GetStore().Add(&corev1.Secret{
-		ObjectMeta: metav1.ObjectMeta{
-			Name:      tokenName,
-			Namespace: ns,
-			Annotations: map[string]string{
-				corev1.ServiceAccountNameKey: serviceAccountName,
-				corev1.ServiceAccountUIDKey:  serviceAccountUID,
-			},
-		},
-		Type: corev1.SecretTypeServiceAccountToken,
-		Data: map[string][]byte{
-			api.ServiceAccountTokenKey: []byte("token-data"),
-		},
-	})
-
-	pod := &api.Pod{
-		Spec: api.PodSpec{
-			ServiceAccountName: serviceAccountName,
-			Containers: []api.Container{
-				{Name: "container-1"},
-			},
-		},
-	}
-
-	attrs := admission.NewAttributesRecord(pod, nil, api.Kind("Pod").WithVersion("version"), ns, "myname", api.Resource("pods").WithVersion("version"), "", admission.Create, &metav1.CreateOptions{}, false, nil)
-	if err := admissiontesting.WithReinvocationTesting(t, admit).Admit(context.TODO(), attrs, nil); err != nil {
-		t.Fatal(err)
-	}
-
-	if len(pod.Spec.Volumes) != 1 {
-		t.Fatalf("Expected 1 volume, got %d", len(pod.Spec.Volumes))
-	}
-	if !reflect.DeepEqual(expectedVolume, pod.Spec.Volumes[0]) {
-		t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolume, pod.Spec.Volumes[0])
-	}
-	if len(pod.Spec.Containers[0].VolumeMounts) != 1 {
-		t.Fatalf("Expected 1 volume mount, got %d", len(pod.Spec.Containers[0].VolumeMounts))
-	}
-	if !reflect.DeepEqual(expectedVolumeMount, pod.Spec.Containers[0].VolumeMounts[0]) {
-		t.Fatalf("Expected\n\t%#v\ngot\n\t%#v", expectedVolumeMount, pod.Spec.Containers[0].VolumeMounts[0])
-	}
-}
-
 func testGenerateName(n string) string {
 	return n + "abc123"
 }
 
 var generatedVolumeName = testGenerateName(ServiceAccountVolumeName + "-")
-
-func testBoundServiceAccountTokenVolumePhases(t *testing.T, f func(*testing.T, func(*Plugin) *Plugin)) {
-	t.Run("BoundServiceAccountTokenVolume disabled", func(t *testing.T) {
-		f(t, func(s *Plugin) *Plugin {
-			s.boundServiceAccountTokenVolume = deprecationDisabledBoundTokenVolume
-			return s
-		})
-	})
-
-	t.Run("BoundServiceAccountTokenVolume enabled", func(t *testing.T) {
-		f(t, func(s *Plugin) *Plugin {
-			s.boundServiceAccountTokenVolume = deprecationEnabledBoundTokenVolume
-			return s
-		})
-	})
-}
